#pragma once

#include "rmm.h"
#include "FrontEndBase.h"
#include "vmm_tbl_regs_map.h"
#include "vmm_config.h"

/* Struct to hold Hyb, VMM pair values */
struct Pair {
  int hyb = -1;
  int vmm = -1;
};

/** Class definition for the VMMTbl class */
class VMMTbl : public FrontEndBase {
public:
  VMMTbl(RMM& rmm, int ring, int node, int hybrids);
  asynStatus createEpicsParams();
	virtual asynStatus writeInt32(asynUser *pasynUser, epicsInt32 value);
  virtual asynStatus writeFloat64(asynUser *pasynUser, epicsFloat64 value);
  virtual asynStatus writeInt8Array(asynUser *pasynUser, epicsInt8 *value, size_t nElements);
  void setInitialEpicsParams();
  void configFE();
  Pair findPair(int function, int array[HYBRIDS_PER_FEN][VMMS_PER_HYBRID]);

protected:
  int vmmSelectMonitorVMM0_;
  int vmmSelectMonitorVMM1_;
  int vmmProcessReadMonitorVMMs_;
  int vmmMonitorValueVMM0_;
  int vmmMonitorValueVMM1_;
  int vmmAcquire_;
  int vmmHybFwVersion_[HYBRIDS_PER_FEN];
  int vmmHybId_[HYBRIDS_PER_FEN];
  int vmmHybGeoPos_[HYBRIDS_PER_FEN];
  int vmmHybLinkStatus_[HYBRIDS_PER_FEN];
  //Registers channels per hybrid, per VMM
  int vmmST_[HYBRIDS_PER_FEN][VMMS_PER_HYBRID];

private:
  static constexpr const char *driverName = "VMMTbl";
  FENConfigModule pVMMConfig;
  RMM *pRMM;
  int addr_list = 0;
  int number_hybrids;
};
